require "defines"
require "harvester"
require "refinery"
require "migrations"

remote.addinterface("harvester", {
	Error = function()
		Error()
	end,
})

game.oninit(function() OnLoad() end)
game.onload(function() OnLoad() end)
game.onevent(defines.events.ontick, function() OnTick() end)

game.onevent(defines.events.onbuiltentity, function(event) OnBuilt(event) end)
game.onevent(defines.events.onrobotbuiltentity, function(event) OnBuilt(event) end)

game.onevent(defines.events.onpreplayermineditem, function(event) OnRemoved(event) end)
game.onevent(defines.events.onrobotpremined, function(event) OnRemoved(event) end)
game.onevent(defines.events.onentitydied, function(event) OnRemoved(event) end)

function OnLoad()
	if not glob.harvesters then
		glob.harvesters = {}
	else
		for _, harvester in pairs(glob.harvesters) do
			-- Allow it to re-set its metatable.
			Harvester.Onload(harvester)
		end
	end
	
	if not glob.refineries then
		glob.refineries = {}
	else
		for _, refinery in pairs(glob.refineries) do
			-- Allow it to re-set its metatable.
			Refinery.Onload(refinery)
		end
	end
	
	CheckVersion()
end

function OnTick()
	for _, harvester in pairs(glob.harvesters) do
		harvester:Tick()
	end
	for _, refinery in pairs(glob.refineries) do
		refinery:Tick()
	end
end

function OnBuilt(event)
	local ent = event.createdentity
	if ent.name == "harvester" then
		table.insert(glob.harvesters, Harvester.New(ent))
	end
	if ent.name == "refinery" then
		table.insert(glob.refineries, Refinery.New(ent))
	end
end

function OnRemoved(event)
	local ent = event.entity
	if ent.name == "harvester" then
		for i, harvester in pairs(glob.harvesters) do
			if harvester.vehicle.equals(ent) then
				harvester:Delete()
				table.remove(glob.harvesters, i)
				return
			end
		end
	elseif ent.name == "harvester-anim" then
		for i, harvester in pairs(glob.harvesters) do
			if harvester.animEntity.equals(ent) then
				harvester:Delete()
				table.remove(glob.harvesters, i)
				return
			end
		end
	elseif ent.name == "refinery" then
		for i, refinery in pairs(glob.refineries) do
			if refinery.refinery.equals(ent) then
				refinery:Delete()
				table.remove(glob.refineries, i)
				return
			end
		end
	elseif ent.name == "refinery-chest" then
		for i, refinery in pairs(glob.refineries) do
			if refinery.chest.equals(ent) then
				refinery:Delete()
				table.remove(glob.refineries, i)
				return
			end
		end
	end
end

function CheckVersion()
	if not glob.harvesterVersion then
		for _, player in pairs(game.players) do
			player.print("Harvesters: Migrating to version " .. currentVersion)
		end
		migrations["unversioned"]()
	elseif glob.harvesterVersion ~= currentVersion then
		if migrations[glob.harvesterVersion] then
			for _, player in pairs(game.players) do
				game.player.print("Harvesters: Migrating from version " .. glob.harvesterVersion .. " to version " .. currentVersion)
			end
			migrations[glob.harvesterVersion]()
		end
	end
	glob.harvesterVersion = currentVersion
end